@@ -1,6 +1,7 @@
 import re
 from abc import ABC, abstractmethod
 from contextlib import nullcontext
+from typing import Callable, List
 
 import torch
 
@@ -11,13 +12,33 @@ from .features import FeaturesAndLogitsHook
 
 
 class BaseReducer(BaseHook, ABC):
-    def __init__(self, apply_to=None, default_reducer=None, **kwargs):
+    """
+    Converts an unreduced loss tensor into a single number.
+    In other words, if the loss tensor has shape ```(N,)```,
+    the reducer converts it to shape ```(1,)```.
+    """
+
+    def __init__(
+        self,
+        apply_to: List[str] = None,
+        default_reducer: "BaseReducer" = None,
+        **kwargs,
+    ):
+        """
+        Arguments:
+            apply_to: list of loss names to apply reduction to
+            default_reducer: a reducer to use for losses that
+                are not already reduced and are also not
+                specified in ```apply_to```. If ```None```,
+                then no action is taken.
+        """
         super().__init__(**kwargs)
         self.apply_to = apply_to
         self.default_reducer = default_reducer
         self.curr_loss_keys = []
 
     def call(self, losses, inputs):
+        """"""
         self.curr_loss_keys = list(losses.keys())
         apply_to = self.get_keys_to_apply_to(losses)
         losses, outputs = self.call_reducer(losses, inputs, apply_to)
@@ -36,6 +57,7 @@ class BaseReducer(BaseHook, ABC):
         pass
 
     def _loss_keys(self):
+        """"""
         return self.curr_loss_keys
 
     def get_keys_to_apply_to(self, losses):
@@ -70,14 +92,32 @@ class MultipleReducers(BaseHook):
 
 
 class EntropyReducer(BaseReducer):
+    """
+    Implementation of "entropy conditioning" from
+    [Conditional Adversarial Domain Adaptation](https://arxiv.org/abs/1705.10667).
+    It weights loss elements using
+    [```EntropyWeights```][pytorch_adapt.layers.entropy_weights.EntropyWeights].
+    The entropy weights are derived from classifier logits.
+    """
+
     def __init__(
         self,
-        f_hook=None,
-        domains=None,
-        entropy_weights_fn=None,
-        detach_weights=True,
+        f_hook: BaseHook = None,
+        domains: List[str] = None,
+        entropy_weights_fn: Callable[[torch.Tensor], torch.Tensor] = None,
+        detach_weights: bool = True,
         **kwargs,
     ):
+        """
+        Arguments:
+            f_hook: the hook for computing logits from
+                which entropy weights are derived
+            domains: the domains that ```f_hook``` should compute for
+            entropy_weights_fn: the function for computing the weights
+                that will be multiplied with the unreduced losses.
+            detach_weights: If ```True```, the entropy weights are
+                detached from the autograd graph
+        """
         super().__init__(**kwargs)
         src_regex = "^{0}_|_{0}$|_{0}_|^{0}$".format("src")
         target_regex = "^{0}_|_{0}$|_{0}_|^{0}$".format("target")
@@ -113,14 +153,20 @@ class EntropyReducer(BaseReducer):
         return losses, outputs
 
     def _out_keys(self):
+        """"""
         return self.f_hook.out_keys
 
 
 class MeanReducer(BaseReducer):
+    """
+    Reduces loss elements by taking the mean.
+    """
+
     def call_reducer(self, losses, inputs, apply_to):
         for k in apply_to:
             losses[k] = torch.mean(losses[k])
         return losses, {}
 
     def _out_keys(self):
+        """"""
         return []
